@@ -6,18 +6,19 @@ import re
 from datetime import datetime
 from exceptions import FutureWarning
 from warnings import warn
-from urlparse import urlparse
 
 import requests
 from http_signature.requests_auth import HTTPSignatureAuth
 
 from .machine import Machine
+from ._version import get_versions
+__version__ = get_versions()['version']
+del get_versions
 
-__all__ = ['DataCenter', 'DEBUG_CONFIG', 'KNOWN_LOCATIONS', 
+__all__ = ['DataCenter', 'KNOWN_LOCATIONS', 
             'TELEFONICA_LOCATIONS', 'DEFAULT_LOCATION']
 
 API_HOST_SUFFIX = '.api.joyentcloud.com'
-API_VERSION = '~6.5'
 
 KNOWN_LOCATIONS = {
     u'us-east-1': u'https://us-east-1.api.joyentcloud.com',
@@ -38,12 +39,9 @@ DEFAULT_LOCATION = 'us-west-1'
 DEFAULT_HEADERS = {
     'Accept':        'application/json',
     'Content-Type':  'application/json; charset=UTF-8',
-    'X-Api-Version':  API_VERSION,
-    'User-Agent':    'py-smartdc'
+    'User-Agent':    'py-smartdc (%s)' % __version__
 }
 
-DEBUG_CONFIG = {'verbose': sys.stderr}
-
 
 def search_dicts(dicts, predicate, fields):
     matcher = re.compile(predicate, re.IGNORECASE)
@@ -64,8 +62,10 @@ class DataCenter(object):
     requests it, and only accesses the REST API on method calls (never on 
     attribute access).
     """
+    API_VERSION = '7.0'
+    
     def __init__(self, location=None, key_id=None, secret='~/.ssh/id_rsa', 
-                headers=None, login=None, config=None, known_locations=None,
+                headers=None, login=None, known_locations=None,
                 allow_agent=False, verify=True, verbose=None):
         """
         A :py:class:`smartdc.datacenter.DataCenter` object may be instantiated 
@@ -88,9 +88,6 @@ class DataCenter(object):
         :param login: user path in SmartDC
         :type login: :py:class:`basestring`
         
-        :param config: Requests-style configuration [deprecated]
-        :type config: :py:class:`dict`
-        
         :param known_locations: keys-to-URLs mapping used by `location` 
         :type known_locations: :py:class:`dict`
         
@@ -121,12 +118,7 @@ class DataCenter(object):
         """
         self.location = location or DEFAULT_LOCATION
         self.known_locations = known_locations or KNOWN_LOCATIONS
-        if verbose is None and config and config.get('verbose'):
-            self.verbose = config.get('verbose')
-            warn("DataCenter: config= will be eliminated in "
-                "favor of verbose=", FutureWarning, stacklevel=2)
-        else:
-            self.verbose = verbose and sys.stderr
+        self.verbose = verbose and sys.stderr
         self.verify = verify
         if key_id and secret:
             self.auth = HTTPSignatureAuth(key_id=key_id, secret=secret,
@@ -134,6 +126,7 @@ class DataCenter(object):
         else:
             self.auth = None
         self.default_headers = DEFAULT_HEADERS
+        self.default_headers['X-Api-Version'] = self.API_VERSION
         if headers:
             self.default_headers.update(headers)
         if login:
@@ -192,9 +185,6 @@ class DataCenter(object):
         """Protocol + hostname"""
         if self.location in self.known_locations:
             return self.known_locations[self.location]
-        loc = urlparse(self.location)
-        if loc.scheme.startswith('http'):
-            return self.location
         elif '.' in self.location or self.location == 'localhost':
             return 'https://' + self.location
         else:
@@ -424,33 +414,14 @@ class DataCenter(object):
         else:
             return j
     
-    def default_dataset(self):
-        """
-        ::
-        
-            GET /:login/datasets
-        
-        :Returns: the default dataset for this datacenter
-        :rtype: :py:class:`dict` or ``None``
-        
-        Requests all the datasets in this datacenter, filters for the default, 
-        and returns the corresponding :py:class:`dict`, if a default has been 
-        defined.
-        """
-        sets = filter(itemgetter('default'), self.datasets())
-        if sets:
-            return sets[0]
-        else:
-            return None
-    
-    def dataset(self, dataset_id):
+    def dataset(self, identifier):
         """
         ::
         
             GET /:login/datasets/:id
         
-        :param dataset_id: unique ID or URN for a dataset
-        :type dataset_id: :py:class:`basestring` or :py:class:`dict`
+        :param identifier: unique ID or URN for a dataset
+        :type identifier: :py:class:`basestring` or :py:class:`dict`
         
         :rtype: :py:class:`dict`
         
@@ -458,34 +429,60 @@ class DataCenter(object):
         also prefix-matched. If passed a dict that contains an `urn` or `id` 
         key, it uses the respective value as the identifier.
         """
-        if isinstance(dataset_id, dict):
-            dataset_id = dataset_id.get('urn', dataset_id['id'])
-        j, _ = self.request('GET', 'datasets/' + str(dataset_id))
+        if isinstance(identifier, dict):
+            identifier = identifier.get('id', identifier['urn'])
+        j, _ = self.request('GET', 'datasets/' + str(identifier))
         return j
-    
-    def packages(self, search=None, fields=('name',)):
+
+    def packages(self, name=None, memory=None, disk=None, swap=None,
+                version=None, vcpus=None, group=None):
         """
         ::
         
             GET /:login/packages
         
-        :param search: optionally filter (locally) with a regular expression 
-            search on the listed fields
-        :type search: :py:class:`basestring` that compiles as a regular 
-            expression
+        :param name: the label associated with the resource package
+        :type name: :py:class:`basestring`
         
-        :param fields: filter on the listed fields (defaulting to ``name``\)
-        :type fields: :py:class:`list` of :py:class:`basestring`\s
+        :param memory: amount of RAM (in MiB) that the package provisions
+        :type memory: :py:class:`int`
+        
+        :param disk: amount of disk storage (in MiB) the package provisions
+        :type disk: :py:class:`int`
+        
+        :param swap: amount of swap (in MiB) the package provisions
+        :type swap: :py:class:`int`
+        
+        :param version: the version identifier associated with the package
+        :type version: :py:class:`basestring`
+        
+        :param vcpus: the number of virtual CPUs provisioned with the package
+        :type vcpus: :py:class:`int`
+        
+        :param group: the group to which the package belongs
+        :type group: :py:class:`basestring`
         
         :Returns: packages (machine "sizes", with resource types and values) 
             available in this datacenter.
         :rtype: :py:class:`list` of :py:class:`dict`\s
         """
-        j, _ = self.request('GET', 'packages')
-        if search:
-            return list(search_dicts(j, search, fields))
-        else:
-            return j
+        params = {}
+        if name:
+            params['name'] = name
+        if memory:
+            params['memory'] = memory
+        if disk:
+            params['disk'] = disk
+        if swap:
+            params['swap'] = swap
+        if version:
+            params['version'] = version
+        if vcpus:
+            params['vcpus'] = vcpus
+        if group:
+            params['group'] = group
+        j, _ = self.request('GET', 'packages', params=params)
+        return j
     
     def default_package(self):
         """
@@ -499,7 +496,8 @@ class DataCenter(object):
         Requests all the packages in this datacenter, filters for the default, 
         and returns the corresponding dict, if a default has been defined.
         """
-        packages = filter(itemgetter('default'), self.packages())
+        packages = [pk for pk in self.packages() 
+                        if pk.get('default') == 'true']
         if packages:
             return packages[0]
         else:
@@ -521,7 +519,7 @@ class DataCenter(object):
         value.
         """
         if isinstance(name, dict):
-            name = name['name']
+            name = name.get('id', name.get('name'))
         j, _ = self.request('GET', 'packages/' + str(name))
         return j
     
@@ -684,7 +682,8 @@ class DataCenter(object):
         return [Machine(datacenter=self, data=m) for m in machines]
     
     def create_machine(self, name=None, package=None, dataset=None,
-            metadata=None, tags=None, boot_script=None, credentials=False):
+            metadata=None, tags=None, boot_script=None, credentials=False,
+            image=None, networks=None):
         """
         ::
         
@@ -702,8 +701,12 @@ class DataCenter(object):
         :param package: cluster of resource values identified by name
         :type package: :py:class:`basestring` or :py:class:`dict`
         
+        :param image: an identifier for the base operating system image
+            (formerly a ``dataset``)
+        :type image: :py:class:`basestring` or :py:class:`dict`
+        
         :param dataset: base operating system image identified by a globally 
-            unique ID or URN
+            unique ID or URN (deprecated)
         :type dataset: :py:class:`basestring` or :py:class:`dict`
         
         :param metadata: keys & values with arbitrary supplementary 
@@ -713,29 +716,37 @@ class DataCenter(object):
         :param tags: keys & values with arbitrary supplementary 
             identifying information for filtering when querying for machines
         :type tags: :py:class:`dict`
+
+        :param networks: list of networks where this machine will belong to
+        :type networks: :py:class:`list`
         
         :param boot_script: path to a file to upload for execution on boot
         :type boot_script: :py:class:`basestring` as file path
         
         :rtype: :py:class:`smartdc.machine.Machine`
         
-        If `package` or `dataset` are passed a :py:class:`dict` containing a 
-        `name` key (in the case of `package`) or a `urn` or `id` key (in the 
-        case of `dataset`), it passes the corresponding value. The server API 
+        If `package`, `image`, or `dataset` are passed a :py:class:`dict` containing a 
+        `name` key (in the case of `package`) or an `id` key (in the case of
+        `image` or `dataset`), it passes the corresponding value. The server API 
         appears to resolve incomplete or ambiguous dataset URNs with the 
         highest version number.
         """
         params = {}
         if name:
-            assert re.match(r'[a-zA-Z0-9]([a-zA-Z0-9\-\.]*[a-zA-Z0-9])?$', name), "Illegal name"
+            assert re.match(r'[a-zA-Z0-9]([a-zA-Z0-9\-\.]*[a-zA-Z0-9])?$',
+                name), "Illegal name"
             params['name'] = name
         if package:
             if isinstance(package, dict):
                 package = package['name']
             params['package'] = package
-        if dataset:
+        if image:
+            if isinstance(image, dict):
+                image = image['id']
+            params['image'] = image
+        if dataset and not image:
             if isinstance(dataset, dict):
-                dataset = dataset.get('urn', dataset['id'])
+                dataset = dataset.get('id', dataset['urn'])
             params['dataset'] = dataset
         if metadata:
             for k, v in metadata.items():
@@ -746,6 +757,11 @@ class DataCenter(object):
         if boot_script:
             with open(boot_script) as f:
                 params['metadata.user-script'] = f.read()
+        if networks:
+            if isinstance(networks, list):
+                params['networks'] = networks
+            elif isinstance(networks, basestring):
+                params['networks'] = [networks]
         j, r = self.request('POST', 'machines', data=params)
         if r.status_code >= 400:
             print(j, file=sys.stderr)
@@ -771,4 +787,101 @@ class DataCenter(object):
         return Machine(datacenter=self, machine_id=machine_id, 
                 credentials=credentials)
     
-
+    def networks(self, search=None, fields=('name,')):
+        """
+        ::
+        
+            GET /:login/networks
+        
+        :param search: optionally filter (locally) with a regular expression
+            search on the listed fields
+        :type search: :py:class:`basestring` that compiles as a regular
+            expression
+        
+        :param fields: filter on the listed fields (defaulting to
+            ``name``)
+        :type fields: :py:class:`list` of :py:class:`basestring`\s
+        
+        :Returns: network available in this datacenter
+        :rtype: :py:class:`list` of :py:class:`dict`\s
+        """
+        
+        j, _ = self.request('GET', 'networks')
+        if search:
+            return list(search_dicts(j, search, fields))
+        else:
+            return j
+    
+    def network(self, identifier):
+        """
+        ::
+        
+            GET /:login/networks/:id
+        
+        :param identifier: match on the listed network identifier
+        :type identifier: :py:class:`basestring` or :py:class:`dict`
+        
+        :Returns: characteristics of the requested network
+        :rtype: :py:class:`dict`
+        
+        Either a string or a dictionary with an ``id`` key may be passed in.
+        """
+        
+        if isinstance(identifier, dict):
+            identifier = identifier.get('id')
+        j, _ = self.request('GET', 'networks/' + str(identifier))
+        return j
+    
+    def images(self, name=None, os=None, version=None):
+        """
+        ::
+        
+            GET /:login/images
+        
+        :param name: match on the listed name
+        :type name: :py:class:`basestring`
+        
+        :param os: match on the selected os
+        :type os: :py:class:`basestring`
+        
+        :param version: match on the selected version
+        :type version: :py:class:`basestring`
+        
+        :Returns: available machine images in this datacenter
+        :rtype: :py:class:`list` of :py:class:`dict`\s
+        """
+        
+        params = {}
+        if name:
+            params['name'] = name
+        if os:
+            params['os'] = os
+        if version:
+            params['version'] = version
+        j, _ = self.request('GET', 'images', params=params)
+        
+        return j
+    
+    def image(self, identifier):
+        """
+        ::
+        
+            GET /:login/images/:id
+        
+        :param identifier: match on the listed image identifier
+        :type identifier: :py:class:`basestring` or :py:class:`dict`
+        
+        :Returns: the details of the given image matching the identifier
+        :rtype: :py:class:`dict`
+        
+        A string or a dictionary containing an ``id`` key may be
+        passed in.
+        """
+        
+        if isinstance(identifier, dict):
+            identifier = identifier.get('id', '')
+        j, _ = self.request('GET', 'images/' + str(identifier))
+        
+        return j
+    
+    
\ No newline at end of file
